@@ -34,7 +34,10 @@ instruction_dtype = [(('Waveform simulator event number.', 'event_number'), np.i
                      (('Volume id giving the detector subvolume', 'vol_id'), np.int32),
                      (('Local field [ V / cm ]', 'local_field'), np.float64),
                      (('Number of excitons', 'n_excitons'), np.int32),
-]
+                     (('X position of the primary particle [cm]', 'x_pri'), np.float32),
+                     (('Y position of the primary particle [cm]', 'y_pri'), np.float32),
+                     (('Z position of the primary particle [cm]', 'z_pri'), np.float32),
+                    ]
 
 
 optical_extra_dtype = [(('first optical input index', '_first'), np.int32),
@@ -68,13 +71,56 @@ truth_extra_dtype = [
     (('Standard deviation of electron arrival times [ns]', 't_sigma_electron'), np.float64),]
 
 
+@export
+def extra_truth_dtype_per_pmt(n_pmt: ty.Union[bool, int]) -> ty.List[tuple]:
+    """
+    Generate the extra - truth dtype output
+
+    Return total/bottom seperation from truth_extra_dtype if n_pmt is False
+
+    :param n_pmt: Number of PMTs, when false (no PMTs, use total/bottom separation)
+    :return: dtype list
+    """
+    if not n_pmt:
+        return truth_extra_dtype
+    return [
+        (('End time of the interaction [ns]', 'endtime'), np.int64),
+        (('Number of simulated electrons', 'n_electron'), np.int32),
+        # Per PMT fields
+        (('Number of photons reaching PMT', 'n_photon_per_pmt'), (np.int32, n_pmt)),
+        (('Number of photons + dpe passing', 'n_pe_per_pmt'), (np.int32, n_pmt)),
+        (('Number of photons passing trigger', 'n_photon_trigger_per_pmt'), (np.int32, n_pmt)),
+        (('Number of photons + dpe passing trigger', 'n_pe_trigger_per_pmt'), (np.int32, n_pmt)),
+        (('Raw area in pe', 'raw_area_per_pmt'), (np.float64, n_pmt)),
+        (('Raw area in pe passing trigger', 'raw_area_trigger_per_pmt'), (np.float64, n_pmt)),
+        # Aggregate (total) fields
+        (('Number of photons reaching PMT (total)', 'n_photon'), np.int32),
+        (('Number of photons + dpe passing (total)', 'n_pe'), np.int32),
+        (('Number of photons passing trigger (total)', 'n_photon_trigger'), np.int32),
+        (('Number of photons + dpe passing trigger (total)', 'n_pe_trigger'), np.int32),
+        (('Raw area in pe (total)', 'raw_area'), np.float64),
+        (('Raw area in pe passing trigger (total)', 'raw_area_trigger'), np.float64),
+        # General fields
+        (('Arrival time of the first photon [ns]', 't_first_photon'), np.float64),
+        (('Arrival time of the last photon [ns]', 't_last_photon'), np.float64),
+        (('Mean time of the photons [ns]', 't_mean_photon'), np.float64),
+        (('Standard deviation of photon arrival times [ns]', 't_sigma_photon'), np.float64),
+        (('X field-distorted mean position of the electrons [cm]', 'x_mean_electron'), np.float32),
+        (('Y field-distorted mean position of the electrons [cm]', 'y_mean_electron'), np.float32),
+        (('Arrival time of the first electron [ns]', 't_first_electron'), np.float64),
+        (('Arrival time of the last electron [ns]', 't_last_electron'), np.float64),
+        (('Mean time of the electrons [ns]', 't_mean_electron'), np.float64),
+        (('Standard deviation of electron arrival times [ns]', 't_sigma_electron'), np.float64),
+    ]
+
+
 def rand_instructions(c):
     """Random instruction generator function. This will be called by wfsim if you do not specify 
     specific instructions.
     :params c: wfsim configuration dict"""
-    log.warn('rand_instructions is deprecated, please use wfsim.random_instructions')
+    log.warning('rand_instructions is deprecated, please use wfsim.random_instructions')
     if 'drift_field' not in c:
-        log.warn('drift field not specified!')
+        log.warning('drift field not specified!')
     # Do the conversion for now, don't break everything all at once
     return _rand_instructions(event_rate=c.get('event_rate', 10),
                               chunk_size=c.get('chunk_size', 5),
@@ -138,6 +184,10 @@ def _rand_instructions(
     inst['y'] = np.repeat(r * np.sin(t), 2)
     inst['z'] = np.repeat(np.random.uniform(-tpc_length, 0, n_events), 2)
 
+    inst['x_pri'] = inst['x']
+    inst['y_pri'] = inst['y']
+    inst['z_pri'] = inst['z']
+
     # Here we'll define our XENON-like detector
     nest_calc = nestpy.NESTcalc(nestpy.VDetector())
     nucleus_A = 131.293
@@ -175,7 +225,7 @@ def _rand_instructions(
     inst['e_dep'] = e_dep
     for field in inst.dtype.names:
         if np.any(inst[field] == -1):
-            log.warn(f'{field} is not (fully) filled')
+            log.warning(f'{field} is not (fully) filled')
     return inst
 
 
@@ -185,7 +235,7 @@ def _read_optical_nveto(config, events, mask):
     :params events: g4 root file
     :params mask: 1d bool array to select events
     
-    returns two flatterned nested arrays of channels and timings, 
+    returns two flatterned nested arrays of channels and timings,
     """
     channels = np.hstack(events["pmthitID"].array(library="np")[mask])
     timings = np.hstack(events["pmthitTime"].array(library="np")[mask] * 1e9).astype(np.int64)
@@ -265,6 +315,9 @@ def read_optical(config):
     ins['x'] = events["xp_pri"].array(library="np").flatten()[mask] / 10.
     ins['y'] = events["yp_pri"].array(library="np").flatten()[mask] / 10.
     ins['z'] = events["zp_pri"].array(library="np").flatten()[mask] / 10.
+    ins['x_pri'] = np.zeros(n_events, np.int64)
+    ins['y_pri'] = np.zeros(n_events, np.int64)
+    ins['z_pri'] = np.zeros(n_events, np.int64)
     ins['time'] = np.zeros(n_events, np.int64)
     ins['event_number'] = np.arange(n_events)
     ins['g4id'] = events['eventid'].array(library="np")[mask]
@@ -304,7 +357,9 @@ class ChunkRawRecords(object):
         self.rawdata = rawdata_generator(self.config, **kwargs)
         self.record_buffer = np.zeros(5000000,
                                       dtype=strax.raw_record_dtype(samples_per_record=strax.DEFAULT_RECORD_LENGTH))
-        self.truth_buffer = np.zeros(10000, dtype=instruction_dtype + truth_extra_dtype + [('fill', bool)])
+        truth_per_n_pmts = self._n_channels if config.get('per_pmt_truth') else False
+        self.truth_dtype = extra_truth_dtype_per_pmt(truth_per_n_pmts)
+        self.truth_buffer = np.zeros(10000, dtype=instruction_dtype + self.truth_dtype + [('fill', bool)])
 
         self.blevel = 0  # buffer_filled_level
 
@@ -418,7 +473,7 @@ class ChunkRawRecords(object):
 
         truth.sort(order='time')
         # Return truth without 'fill' field
-        _truth = np.zeros(len(truth), dtype=instruction_dtype + truth_extra_dtype)
+        _truth = np.zeros(len(truth), dtype=instruction_dtype + self.truth_dtype)
         for name in _truth.dtype.names:
             _truth[name] = truth[name]
         _truth['time'][~np.isnan(_truth['t_first_photon'])] = \
@@ -442,6 +497,9 @@ class ChunkRawRecords(object):
     def source_finished(self):
         return self.rawdata.source_finished
 
+    @property
+    def _n_channels(self):
+        return len(self.config.get('gains', np.arange(straxen.n_tpc_pmts)))
 
 @strax.takes_config(
     strax.Option('detector', default='XENONnT', track=True, infer_type=False),
@@ -451,6 +509,8 @@ class ChunkRawRecords(object):
                  help="Duration of each chunk in seconds"),
     strax.Option('n_chunk', default=10, track=False, infer_type=False,
                  help="Number of chunks to simulate"),
+    strax.Option('per_pmt_truth', default=False, track=True, type=bool,
+                 help="Store the info per channel in the truth file"),
     strax.Option('fax_file', default=None, track=False, infer_type=False,
                  help="Directory with fax instructions"), 
     strax.Option('fax_config', default='fax_config_nt_design.json'),
@@ -474,6 +534,7 @@ class ChunkRawRecords(object):
                       "generation of the instructions"),
 )
 class SimulatorPlugin(strax.Plugin):
+    compressor = 'zstd'
     depends_on = tuple()
 
     # Cannot arbitrarily rechunk records inside events
@@ -586,6 +647,14 @@ class SimulatorPlugin(strax.Plugin):
         """Return whether all instructions has been used."""
         return self.sim.source_finished()
 
+    @property
+    def _n_channels(self):
+        return len(self.config.get('gains', np.arange(straxen.n_tpc_pmts)))
+
+    @property
+    def _truth_dtype(self):
+        truth_per_n_pmts = self._n_channels if self.config.get('per_pmt_truth') else False
+        return extra_truth_dtype_per_pmt(truth_per_n_pmts)
 
 @export
 class RawRecordsFromFaxNT(SimulatorPlugin):
@@ -608,18 +677,20 @@ class RawRecordsFromFaxNT(SimulatorPlugin):
         # Let below cathode S1 instructions pass but remove S2 instructions
         m = (self.instructions['z'] < - self.config['tpc_length']) & (self.instructions['type'] == 2)
         self.instructions = self.instructions[~m]
+        r_instr = np.sqrt(self.instructions['x']**2 + self.instructions['y']**2)
 
-        assert np.all(self.instructions['x']**2 + self.instructions['y']**2 < self.config['tpc_radius']**2), \
-            "Interation is outside the TPC"
+        assert np.all((r_instr<self.config['tpc_radius'])|np.isclose(r_instr,self.config['tpc_radius'])), \
+            "Interaction is outside the TPC (radius)"
         assert np.all(self.instructions['z'] < 0.25), \
-            "Interation is outside the TPC"
+            "Interaction is outside the TPC (in Z)"
         assert np.all(self.instructions['amp'] > 0), \
             "Interaction has zero size"
 
     def infer_dtype(self):
         dtype = {data_type: strax.raw_record_dtype(samples_per_record=strax.DEFAULT_RECORD_LENGTH)
                  for data_type in self.provides if data_type != 'truth'}
-        dtype['truth'] = instruction_dtype + truth_extra_dtype
+
+        dtype['truth'] = instruction_dtype + self._truth_dtype
         return dtype
 
     def compute(self):
@@ -651,7 +722,7 @@ class RawRecordsFromFaxOpticalNT(RawRecordsFromFaxNT):
                                    channels=self.channels,
                                    timings=self.timings,)
         self.sim.truth_buffer = np.zeros(10000, dtype=instruction_dtype + optical_extra_dtype
-                                                + truth_extra_dtype + [('fill', bool)])
+                                                + self._truth_dtype + [('fill', bool)])
         self.sim_iter = self.sim(self.instructions)
         
         
@@ -785,11 +856,11 @@ class RawRecordsFromMcChain(SimulatorPlugin):
             m = (self.instructions_epix['z'] < - self.config['tpc_length']) & (self.instructions_epix['type'] == 2)
             self.instructions_epix = self.instructions_epix[~m]
 
-            assert np.all(self.instructions_epix['x']**2 + self.instructions_epix['y']**2 <
-                          self.config['tpc_radius']**2), \
-                "Interation is outside the TPC"
+            r_instr = np.sqrt(self.instructions_epix['x']**2 + self.instructions_epix['y']**2)
+            assert np.all((r_instr<self.config['tpc_radius'])|np.isclose(r_instr,self.config['tpc_radius'])), \
+                "Interaction is outside the TPC (radius)"
             assert np.all(self.instructions_epix['z'] < 0.25), \
-                "Interation is outside the TPC"
+                "Interaction is outside the TPC (in Z)"
             assert np.all(self.instructions_epix['amp'] > 0), \
                 "Interaction has zero size"
             assert all(self.instructions_epix['g4id'] >= self.config['entry_start'])
@@ -813,7 +884,7 @@ class RawRecordsFromMcChain(SimulatorPlugin):
                                           channels=self.nveto_channels,
                                           timings=self.nveto_timings,)
             self.sim_nv.truth_buffer = np.zeros(10000, dtype=instruction_dtype + optical_extra_dtype
-                                                + truth_extra_dtype + [('fill', bool)])
+                                                + self._truth_dtype + [('fill', bool)])
 
             assert '_first' in self.instructions_nveto.dtype.names, 'Require indexing info in optical ' \
                                                                     'instruction see optical extra dtype'
@@ -825,7 +896,7 @@ class RawRecordsFromMcChain(SimulatorPlugin):
                 progress_bar=True)
 
     def infer_dtype(self):
-        dtype = dict([(data_type, instruction_dtype + truth_extra_dtype) if 'truth' in data_type
+        dtype = dict([(data_type, instruction_dtype + self._truth_dtype) if 'truth' in data_type
                       else (data_type, strax.raw_record_dtype(samples_per_record=strax.DEFAULT_RECORD_LENGTH))
                       for data_type in self.provides])
         return dtype
@@ -882,7 +953,7 @@ class RawRecordsFromMcChain(SimulatorPlugin):
                 # If nv is not one of the targets just return an empty chunk
                 # If there is TPC event, set TPC time for the start and end
                 else:
-                    dummy_dtype = wfsim.truth_extra_dtype if 'truth' in data_type else strax.raw_record_dtype()
+                    dummy_dtype = self._truth_dtype if 'truth' in data_type else strax.raw_record_dtype()
                     if exist_tpc_result:
                         chunk[data_type] = self.chunk(start=self.sim.chunk_time_pre,
                                                 end=self.sim.chunk_time,
@@ -898,7 +969,7 @@ class RawRecordsFromMcChain(SimulatorPlugin):
                                               data=result[data_type],
                                               data_type=data_type)
                 else:
-                    dummy_dtype = wfsim.truth_extra_dtype if 'truth' in data_type else strax.raw_record_dtype()
+                    dummy_dtype = self._truth_dtype if 'truth' in data_type else strax.raw_record_dtype()
                     if exist_nveto_result:
                         chunk[data_type] = self.chunk(start=self.sim_nv.chunk_time_pre,
                                                       end=self.sim_nv.chunk_time,
